@@ -1,5 +1,5 @@
 package Test::Expr;
-our $VERSION = '0.000007';
+our $VERSION = '0.000008';
 
 use 5.012; use warnings;
 use Keyword::Declare;
@@ -15,8 +15,18 @@ sub _trim {
     return $str;
 }
 
-my $PERL_VAR     = qr{ ((?&PerlVariable)) $PPR::GRAMMAR }xms;
-my $PERL_LITERAL = qr{^ (?> (?&PerlString)
+my $PERL_MATCHABLE = qr{ ^ (?&PerlOWS)
+                           (?>
+                               (?&PerlMatch)
+                             | (?&PerlSubstitution)
+                             | (?&PerlTransliteration)
+                           )
+                           $PPR::GRAMMAR
+                       }xms;
+
+my $PERL_VAR      = qr{ ((?&PerlVariable)) $PPR::GRAMMAR }xms;
+
+my $PERL_LITERAL  = qr{^ (?> (?&PerlString)
                           | (?&PerlQuotelikeQR)
                           | (?&PerlQuotelikeQW)
                           | (?&PerlNumber)        )
@@ -26,7 +36,7 @@ my $PERL_LITERAL = qr{^ (?> (?&PerlString)
 
 my $PERL_EXPR = qr{
     ^ (?>(?&PerlOWS))  (?<lhs> (?>(?&PerlHighBinaryExpression)) )
-      (?>(?&PerlOWS))  (?<op>  (?> [<>]=?|[=!]=|<=>|[~=!]=|\b(?:[lg]t|[lgn]e|eq|cmp)\b))
+      (?>(?&PerlOWS))  (?<op>  (?> [<>]=?|[=!]=|<=>|[~=!]~|\b(?:[lg]t|[lgn]e|eq|cmp)\b))
       (?>(?&PerlOWS))  (?<rhs> .*)
 
     (?(DEFINE)
@@ -65,38 +75,57 @@ sub import {
     keyword ok (ListElem $test, Comma, ListElem $desc) {
         # Work out what values to report if there's a problem...
         (my $test_code = $test) =~ s/^\s*do\s*\{(.*)\}\s*$/$1/;
+
+        # Handle low-precedence prefix not...
         (my $pos_test_code = $test_code) =~ s/^\s*(not)\b//;
         my $negative = $pos_test_code eq $test_code ? q{} : 'not';
+
+        # Extract components of test, if possible...
         $pos_test_code =~ $PERL_EXPR;
         my ($lhs, $op, $rhs) = @+{qw<lhs op rhs>};
 
+        # These hold rearranged code to capture and test component values (if possible)...
         my $test_setup;
         my $test_expr;
-        if ($op) {
-            $test_setup = qq{my \$_____l_h_s_____ = $lhs; my \$_____r_h_s_____ = $rhs;};
-            $test_expr  = $negative . ' $_____l_h_s_____ '.$op.' $_____r_h_s_____';
+
+        # Don't try to capture explicit regex matches on the rhs...
+        if ($op && $op =~ /[=!~]~/ && $rhs =~ $PERL_MATCHABLE) {
+            $test_setup = qq{ my \$_____l_h_s_____ = $lhs; };
+            $test_expr  = qq{ $negative \$_____l_h_s_____ $op $rhs };
+            $rhs = "";
         }
+
+        # Catch lhs and rhs values for other comparison operators...
+        elsif ($op) {
+            $test_setup = qq{ my \$_____l_h_s_____ = $lhs; my \$_____r_h_s_____ = $rhs; };
+            $test_expr  = qq{ $negative \$_____l_h_s_____ $op \$_____r_h_s_____ };
+        }
+
+        # Otherwise just execute the text verbatim...
         else {
             $lhs = "";
             $rhs = "";
             $test_setup = qq{my \$_____l_h_s_____ = $test_code;};
-            $test_expr = q{$_____l_h_s_____};
+            $test_expr  = q{$_____l_h_s_____};
         }
 
+        # Extract and tidy all variables in the test expression...
         my @vars = grep {defined} $test =~ m{$PERL_VAR}g;
-
         ($lhs, $rhs, @vars) = map {_trim $_} $lhs, $rhs, @vars;
 
+        # Find maximum width of any reported value, so we can align them...
         my $var_len = max map {length} $lhs, $rhs, @vars;
+
+        # Generate diagnostics (reporting each distinct value only once)...
         my %seen = ( $rhs => 1, $lhs => 1 );
         my @diagnostics = (
               ( $lhs && $lhs !~ $PERL_LITERAL
-                  ? qq{diag sprintf(q{    %${var_len}s --> }, '$lhs'),
+                  ? qq{diag sprintf(q{    %${var_len}s --> }, q{$lhs}),
                       Data::Dump::dump(\$_____l_h_s_____);}
                   : ()
               ),
               ( $rhs && $rhs !~ $PERL_LITERAL
-                  ? qq{diag sprintf(q{    %${var_len}s --> }, '$rhs'),
+                  ? qq{diag sprintf(q{    %${var_len}s --> }, q{$rhs}),
                       Data::Dump::dump(\$_____r_h_s_____);}
                   : ()
               ),
@@ -105,6 +134,7 @@ sub import {
                        @vars
         );
 
+        # Build the diagnostic code...
         unshift @diagnostics, qq{diag q{  because:};}
             if @diagnostics;
         unshift @diagnostics, qq{diag q{}; diag q{ ($test_code) was false};}
@@ -135,7 +165,7 @@ Test::Expr - Test an expression with better error messages
 
 =head1 VERSION
 
-This document describes Test::Expr version 0.000007
+This document describes Test::Expr version 0.000008
 
 
 =head1 SYNOPSIS
